import { GasUtilities } from './utility/GasUtilities';
/** JWT クラス */
export declare class Jwt {
    private gasUtilities;
    private algorithms;
    private requiredPayloadClaims;
    /**
     * Jwtのコンストラクタ
     *
     * @param gasUtilities GasUtilities
     */
    constructor(gasUtilities: GasUtilities);
    /**
     * アクセストークンを作成する
     *
     * @param headerClaim ヘッダークレーム
     * @param payloadClaim ペイロードクレーム
     * @param privateKey 秘密鍵
     * @returns アクセストークンと有効期限
     */
    createAccessToken(headerClaim: any, payloadClaim: any, privateKey: string): any;
    /**
     * ヘッダークレームを作成する
     *
     * @param algorithm 署名アルゴリズム
     * @param jwtType JWTのメディアタイプ
     * @returns ヘッダークレーム
     */
    createHeaderClaim(algorithm: string, jwtType?: string): any;
    /**
     * ペイロードクレームを作成する(exp,iatは自動で追加)
     *
     * @param iss issuer(発行者)
     * @param sub subject(主題)
     * @param aud audience(受信者)
     * @param expireMinuts 有効期間(分)
     * @param privateClaim プライベートクレーム(オプション)
     */
    createPayloadClaim(iss: string, sub: string, aud: string, expireMinuts: number, privateClaim?: any): any;
    /**
     * 署名を作成する
     *
     * @param target 対象文字列
     * @param privateKey 秘密鍵
     * @param algorithm 署名アルゴリズム
     * @returns 署名
     */
    sign(target: string, privateKey: string, algorithm: string): string;
    /**
     * アクセストークンの検証を行う
     *
     * @param privateKey 秘密鍵
     * @param algorithm 署名アルゴリズム
     * @param accessToken アクセストークン
     */
    validate(privateKey: string, algorithm: string, accessToken: string | undefined | null): void;
    /**
     * Base64 Web Safe でデコードしてオブジェクトを生成する
     *
     * @param data 対象データ
     * @returns オブジェクト
     */
    decode(data: string): any;
    /**
     * リフレッシュトークンを作成する
     *
     * @param userSpecificValue ユーザー固有の値
     */
    createRefreshToken(userSpecificValue: string): string;
    /**
     * リフレッシュトークンの有効期限のタイムスタンプを作成する
     *
     * @param effectiveDays 有効日数
     */
    createRefreshTokenExpiryTimeStamp(effectiveDays: number): number;
    /**
     * タイムスタンプを日付形式に変換する
     *
     * @param timeStamp タイムスタンプ
     * @returns 日付形式(yyyy-MM-dd HH:mm:ss)
     */
    timeStampToDateTime(timeStamp: number): string;
    /**
     * 必須ペイロードクレームを設定する
     *
     * @param requiredPayloadClaims 必須ペイロードクレームの配列
     */
    setRequiredPayloadClaims(requiredPayloadClaims: string[]): void;
    /**
     * 必須ペイロードクレームを設取得する
     */
    getRequiredPayloadClaims(): string[];
    /**
     * 数値に前ゼロを付与する
     *
     * @param num 数値
     * @param paddingLength 前ゼロ桁数
     * @returns 前ゼロ付き数値(文字列)
     */
    private paddingZero;
    /**
     * ヘッダークレーム検証を行う
     *
     * @param headerClaim ヘッダークレーム
     */
    private validateHeaderClaim;
    /**
     * ペイロードクレーム検証を行う
     *
     * @param payloadClaim ペイロードクレーム
     */
    private validatePayloadClaim;
}
//# sourceMappingURL=Jwt.d.ts.map