"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Jwt = void 0;
const Message_1 = require("./constant/Message");
/** JWT クラス */
class Jwt {
    /**
     * Jwtのコンストラクタ
     *
     * @param gasUtilities GasUtilities
     */
    constructor(gasUtilities) {
        this.algorithms = ['HS256']; // サポートは HS256 のみ。拡張のため配列にしている。
        this.requiredPayloadClaims = ['iss', 'sub', 'aud', 'exp']; // default
        this.gasUtilities = gasUtilities;
    }
    /**
     * アクセストークンを作成する
     *
     * @param headerClaim ヘッダークレーム
     * @param payloadClaim ペイロードクレーム
     * @param privateKey 秘密鍵
     * @returns アクセストークンと有効期限
     */
    createAccessToken(headerClaim, payloadClaim, privateKey) {
        if (!headerClaim.alg) {
            throw new Error(Message_1.Message.REQUIRED_ALGORITHM_ERROR);
        }
        const encodedHeader = this.gasUtilities.base64EncodeWebSafe(JSON.stringify(headerClaim, null, '\t'));
        const encodedPayload = this.gasUtilities.base64EncodeWebSafe(JSON.stringify(payloadClaim, null, '\t'));
        const signature = this.sign(encodedHeader + '.' + encodedPayload, privateKey, headerClaim.alg);
        return {
            accessToken: encodedHeader + '.' + encodedPayload + '.' + signature,
            expires: payloadClaim.exp,
        };
    }
    /**
     * ヘッダークレームを作成する
     *
     * @param algorithm 署名アルゴリズム
     * @param jwtType JWTのメディアタイプ
     * @returns ヘッダークレーム
     */
    createHeaderClaim(algorithm, jwtType = 'UNKOWN') {
        if (jwtType === 'UNKOWN') {
            return { alg: algorithm };
        }
        return { alg: algorithm, typ: jwtType };
    }
    /**
     * ペイロードクレームを作成する(exp,iatは自動で追加)
     *
     * @param iss issuer(発行者)
     * @param sub subject(主題)
     * @param aud audience(受信者)
     * @param expireMinuts 有効期間(分)
     * @param privateClaim プライベートクレーム(オプション)
     */
    createPayloadClaim(iss, sub, aud, expireMinuts, privateClaim = null) {
        const now = Date.now();
        const expires = new Date(now);
        expires.setMinutes(expires.getMinutes() + expireMinuts);
        const exp = Math.round(expires.getTime() / 1000);
        const iat = Math.round(now / 1000);
        const payload = { iss, sub, aud, exp, iat };
        if (privateClaim) {
            Object.keys(privateClaim).forEach(function (key) {
                payload[key] = privateClaim[key];
            });
        }
        return payload;
    }
    /**
     * 署名を作成する
     *
     * @param target 対象文字列
     * @param privateKey 秘密鍵
     * @param algorithm 署名アルゴリズム
     * @returns 署名
     */
    sign(target, privateKey, algorithm) {
        if (algorithm === 'HS256') {
            const signature = this.gasUtilities.computeHmacSha256Signature(target, privateKey);
            return this.gasUtilities.base64EncodeWebSafe(signature);
        }
        throw new Error(Message_1.Message.NO_SUPPORT_ALGORITHM_ERROR);
    }
    /**
     * アクセストークンの検証を行う
     *
     * @param privateKey 秘密鍵
     * @param algorithm 署名アルゴリズム
     * @param accessToken アクセストークン
     */
    validate(privateKey, algorithm, accessToken) {
        if (!accessToken) {
            throw new Error(Message_1.Message.ACCESS_TOKEN_ERROR);
        }
        if (accessToken.split('.').length !== 3) {
            throw new Error(Message_1.Message.ACCESS_TOKEN_ERROR);
        }
        const [header, payload, signature] = accessToken.split('.');
        const target = header + '.' + payload;
        const validSignature = this.sign(target, privateKey, algorithm);
        // 署名検証
        if (signature !== validSignature) {
            throw new Error(Message_1.Message.ACCESS_TOKEN_ERROR);
        }
        // ヘッダークレーム検証
        const decodedHeader = this.gasUtilities.base64DecodeWebSafeAndToObject(header);
        this.validateHeaderClaim(decodedHeader);
        // ペイロードクレーム検証
        const decodedPayload = this.gasUtilities.base64DecodeWebSafeAndToObject(payload);
        this.validatePayloadClaim(decodedPayload);
    }
    /**
     * Base64 Web Safe でデコードしてオブジェクトを生成する
     *
     * @param data 対象データ
     * @returns オブジェクト
     */
    decode(data) {
        return this.gasUtilities.base64DecodeWebSafeAndToObject(data);
    }
    /**
     * リフレッシュトークンを作成する
     *
     * @param userSpecificValue ユーザー固有の値
     */
    createRefreshToken(userSpecificValue) {
        const rand = Math.random();
        const target = userSpecificValue + String(rand);
        const hash = this.gasUtilities.sha512(target);
        return this.gasUtilities.base64Encode(hash);
    }
    /**
     * リフレッシュトークンの有効期限のタイムスタンプを作成する
     *
     * @param effectiveDays 有効日数
     */
    createRefreshTokenExpiryTimeStamp(effectiveDays) {
        const date = new Date();
        date.setDate(date.getDate() + effectiveDays);
        return Math.round(date.getTime() / 1000);
    }
    /**
     * タイムスタンプを日付形式に変換する
     *
     * @param timeStamp タイムスタンプ
     * @returns 日付形式(yyyy-MM-dd HH:mm:ss)
     */
    timeStampToDateTime(timeStamp) {
        const date = new Date(timeStamp * 1000);
        return (date.getFullYear() +
            '-' +
            this.paddingZero(date.getMonth() + 1, 2) +
            '-' +
            this.paddingZero(date.getDate(), 2) +
            ' ' +
            this.paddingZero(date.getHours(), 2) +
            ':' +
            this.paddingZero(date.getMinutes(), 2) +
            ':' +
            this.paddingZero(date.getSeconds(), 2));
    }
    /**
     * 必須ペイロードクレームを設定する
     *
     * @param requiredPayloadClaims 必須ペイロードクレームの配列
     */
    setRequiredPayloadClaims(requiredPayloadClaims) {
        this.requiredPayloadClaims = [...requiredPayloadClaims];
    }
    /**
     * 必須ペイロードクレームを設取得する
     */
    getRequiredPayloadClaims() {
        return this.requiredPayloadClaims;
    }
    /**
     * 数値に前ゼロを付与する
     *
     * @param num 数値
     * @param paddingLength 前ゼロ桁数
     * @returns 前ゼロ付き数値(文字列)
     */
    paddingZero(num, paddingLength) {
        return ('0'.repeat(paddingLength) + num.toString()).slice(paddingLength * -1);
    }
    /**
     * ヘッダークレーム検証を行う
     *
     * @param headerClaim ヘッダークレーム
     */
    validateHeaderClaim(headerClaim) {
        if (!headerClaim.alg) {
            throw new Error(Message_1.Message.ACCESS_TOKEN_ERROR);
        }
        // アルゴリズム検証
        const filtereAalgorithms = this.algorithms.filter((algorithm) => algorithm === headerClaim.alg);
        if (filtereAalgorithms.length === 0) {
            throw new Error(Message_1.Message.ACCESS_TOKEN_ERROR);
        }
    }
    /**
     * ペイロードクレーム検証を行う
     *
     * @param payloadClaim ペイロードクレーム
     */
    validatePayloadClaim(payloadClaim) {
        // 必須ペイロードクレームチェック
        this.requiredPayloadClaims.forEach((claim) => {
            if (payloadClaim[claim] === undefined) {
                throw new Error(Message_1.Message.ACCESS_TOKEN_ERROR);
            }
        });
        // 有効期限チェック
        const now = Date.now() / 1000;
        const exp = Number(payloadClaim.exp);
        if (exp < now) {
            throw new Error(Message_1.Message.ACCESS_TOKEN_EXPIRED_ERROR);
        }
    }
}
exports.Jwt = Jwt;
